var fs        = require("fs");
var spawn     = require("child_process").spawn;
var path      = require("path");
// FIXME: using my forked lessjs until the pull request is accepted (https://github.com/cloudhead/less.js/pull/997)
var less      = require("./less-fork/lib/less/index.js");
var swig      = require("swig");
var ejs       = require("ejs");
var html      = require("html");
var utils     = require("./utils.js");
var proc      = require("./process.js");

var lessFuns  = require("./less/functions.js");
less.tree.functions["data-url"] = lessFuns.dataUrlFun;

exports.builders = builders = {
	
	less : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, "utf-8", function(error, data) {
			if (error) {
				proc.quit(error);
			};
			
			var dir = path.dirname(inputFilepath);
			var filename = path.basename(inputFilepath);
			var css;
			
			var parser = new(less.Parser)({
				paths		: [dir],
				filename	: filename
			});
			
			parser.parse(data, function(error, tree) {
				if (error) {
					proc.quit(error);
				}
				
				outputFilepath = outputFilepath.replace(/\.less$/i, ".css");
				console.log("LESSing css      : " + outputFilepath);
				
				try {
					css = tree.toCSS({compress : global.config.cssmin});
				}
				catch(e) {
					console.log("=== LESS ERROR (" + e.filename + " " + e.line + "/" + e.index + " ) ============================");
					console.log("    " + e.message);
				}
				
				fs.writeFile(outputFilepath, css, function(error) {
					if (error) {
						proc.quit(error);
					};
					
					// console.log("CREATED: " + outputFilepath);
				});
			});
		});
	},

	js : function(inputFilepath, outputFilepath) {
		var jsp, pro, ast;
		
		fs.readFile(inputFilepath, function(error, data) {
			if (error) {
				console.error(error);
				proc.quit();
			}

			if (global.config.jsmin) {
				console.log("Uglifying js     : " + outputFilepath);
				
				jsp = require("uglify-js").parser;
				pro = require("uglify-js").uglify;

				ast = jsp.parse(data.toString());
				ast = pro.ast_mangle(ast);
				ast = pro.ast_squeeze(ast);
				data = pro.gen_code(ast);
			}

			fs.writeFile(outputFilepath, data, function() {
				// console.log("CREATED: " + outputFilepath);
			});
		});
	},
	
	html : function(inputFilepath, outputFilepath) {
		var tplRoot = global.config.srcDir + "/" + global.config.templateRoot;
		
		swig.init({root : tplRoot});
		
		fs.readFile(inputFilepath, "utf-8", function(error, data) {
			if (error) {
				proc.quit(error);
			};
			
			console.log("Compiling html   : " + outputFilepath);
			var tpl = swig.compileFile(path.relative(tplRoot, inputFilepath));
			/* EATS WHITESPACE! - https://github.com/maxogden/commonjs-html-prettyprinter/issues/3
			var output = html.prettyPrint(tpl.render({ENV:global.config}), {
				indent_size : 1,
				indent_char : "\t",
				unformatted : []
			});
			*/
			
			var output = tpl.render({ENV:global.config});
			
			fs.writeFile(outputFilepath, output, function(error) {
				if (error) {
					proc.quit(error);
				};
				
				// console.log("CREATED: " + outputFilepath);
			});
		});
	},
	
	// DEPRECATED
	ejs : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, "utf-8", function(error, data) {
			if (error) {
				proc.quit(error);
			};
			
			console.log("Compiling ejs    : " + outputFilepath);
			outputFilepath = outputFilepath.replace(/\.ejs$/i, ".html");
			fs.writeFile(outputFilepath, ejs.render(data, {filename : inputFilepath}), function(error) {
				if (error) {
					proc.quit(error);
				};
				
				// console.log("CREATED: " + outputFilepath);
			});
		});
	},
	
	png : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, function(error, data) {
			if (error) {
				console.error(error);
				proc.quit();
			}
			
			var pngout;
			if (global.config.imgmin) {
				fs.stat(inputFilepath, function(err, stats) {
					if (global.config.imgminMaxsize === 0 || stats.size < global.config.imgminMaxsize) {
						pngout = spawn(global.config.binDir + "/pngout.exe", ["/y", inputFilepath, outputFilepath]);
						pngout.stdout.on("data", function(data) {
							console.log("Optimizing image : " + outputFilepath);
						});
						pngout.stderr.on("data", function(data) {
							console.log(data.toString());
						});
					}
				});
			}
			else {
				fs.writeFile(outputFilepath, data, function() {
					// console.log("CREATED: " + outputFilepath);
				});
			}
		});
	},
	
	dir : function(dir) {
		dir && fs.mkdir(dir, function() {
			// console.log("CREATED: " + dir);
		});
	},
	
	other : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, function(error, data) {
			if (error) {
				console.error(error);
				proc.quit();
			}
			
			fs.writeFile(outputFilepath, data, function() {
				// console.log("CREATED: " + outputFilepath);
			});
		});
	}
	
};

exports.build = function(inputDir, outputDir, emptyDir) {
	emptyDir && utils.emptyDir(outputDir);
	_build(inputDir, outputDir);
};

function _build(inputDir, outputDir) {
	utils.readDir(inputDir, function(dir, file, isDir) {
		var inputFilepath = path.normalize(dir + "/" + file);
		var outputFilepath = inputFilepath.replace(inputDir, outputDir);
		var ext = path.extname(inputFilepath).substr(1);
		
		if (isDir) {
			builders.dir(outputFilepath);
		}
		else {
			(builders[ext] || builders.other)(inputFilepath, outputFilepath);
		}
	}, /^_.*$/i);
}
