var fs = require("fs");
var proc = require("./process.js")

exports.readDir = readDir = function(dir, callback, excludePattern) {
	fs.readdir(dir, function(error, files) {
		if (error) {
			console.error(error);
			proc.quit();
		}
		
		files.forEach(function(file) {
			if (excludePattern && excludePattern.test(file)) {
				return;
			}
			
			var filepath = dir + "/" + file;
			var stat = fs.statSync(filepath);
			var isDir = stat.isDirectory(filepath);
			
			callback && callback(dir, file, isDir);
			if (isDir) {
				readDir(filepath, callback, excludePattern);
			}
		});
	});
};

exports.getFiles = getFiles = function(dir, excludePattern) {
	var exists = fs.existsSync(dir);
	var files = [];
	
	if (exists) {
		fs.readdirSync(dir).forEach(function(file) {
			if (excludePattern && excludePattern.test(file)) {
				return;
			}
			
			var filepath = dir + "/" + file;
			var stat = fs.statSync(filepath);
			if (stat.isDirectory()) {
				files = files.concat(getFiles(filepath, excludePattern));
			}
			else {
				files.push(filepath);
			}
		});
	}
	
	return files;
};

exports.getDirs = getDirs = function(dir, excludePattern) {
	var exists = fs.existsSync(dir);
	var dirs = [];
	
	if (exists) {
		fs.readdirSync(dir).forEach(function(file) {
			if (excludePattern && excludePattern.test(file)) {
				return;
			}
			
			var filepath = dir + "/" + file;
			var stat = fs.statSync(filepath);
			if (stat.isDirectory()) {
				dirs.push(filepath);
				dirs = dirs.concat(getDirs(filepath, excludePattern));
			}
		});
	}
	
	return dirs;
};

exports.emptyDir = function(dir) {
	var exists = fs.existsSync(dir);
	var files, dirs;
	if (exists) {
		files = getFiles(dir).reverse();
		dirs = getDirs(dir).reverse();
		
		while (files[0]) {
			fs.unlinkSync(files.shift());
		}
		
		while (dirs[0]) {
			fs.rmdirSync(dirs.shift());
		}
	}
};